# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class AuctionInformation(object):

    """Implementation of the 'Auction Information' model.

    The auction information.

    Attributes:
        auction_site (str): The name of the auction site.
        auction_item_site (str): The auction site URL.
        auction_buyer_id (str): The ID of the buyer who makes the purchase in
            the auction. This ID might be different from the payer ID provided
            for the payment.
        auction_closing_date (str): The date and time, in [Internet date and
            time format](https://tools.ietf.org/html/rfc3339#section-5.6).
            Seconds are required while fractional seconds are optional. Note:
            The regular expression provides guidance but does not reject all
            invalid dates.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "auction_site": 'auction_site',
        "auction_item_site": 'auction_item_site',
        "auction_buyer_id": 'auction_buyer_id',
        "auction_closing_date": 'auction_closing_date'
    }

    _optionals = [
        'auction_site',
        'auction_item_site',
        'auction_buyer_id',
        'auction_closing_date',
    ]

    def __init__(self,
                 auction_site=APIHelper.SKIP,
                 auction_item_site=APIHelper.SKIP,
                 auction_buyer_id=APIHelper.SKIP,
                 auction_closing_date=APIHelper.SKIP):
        """Constructor for the AuctionInformation class"""

        # Initialize members of the class
        if auction_site is not APIHelper.SKIP:
            self.auction_site = auction_site 
        if auction_item_site is not APIHelper.SKIP:
            self.auction_item_site = auction_item_site 
        if auction_buyer_id is not APIHelper.SKIP:
            self.auction_buyer_id = auction_buyer_id 
        if auction_closing_date is not APIHelper.SKIP:
            self.auction_closing_date = auction_closing_date 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        auction_site = dictionary.get("auction_site") if dictionary.get("auction_site") else APIHelper.SKIP
        auction_item_site = dictionary.get("auction_item_site") if dictionary.get("auction_item_site") else APIHelper.SKIP
        auction_buyer_id = dictionary.get("auction_buyer_id") if dictionary.get("auction_buyer_id") else APIHelper.SKIP
        auction_closing_date = dictionary.get("auction_closing_date") if dictionary.get("auction_closing_date") else APIHelper.SKIP
        # Return an object of this model
        return cls(auction_site,
                   auction_item_site,
                   auction_buyer_id,
                   auction_closing_date)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'auction_site={(self.auction_site if hasattr(self, "auction_site") else None)!r}, '
                f'auction_item_site={(self.auction_item_site if hasattr(self, "auction_item_site") else None)!r}, '
                f'auction_buyer_id={(self.auction_buyer_id if hasattr(self, "auction_buyer_id") else None)!r}, '
                f'auction_closing_date={(self.auction_closing_date if hasattr(self, "auction_closing_date") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'auction_site={(self.auction_site if hasattr(self, "auction_site") else None)!s}, '
                f'auction_item_site={(self.auction_item_site if hasattr(self, "auction_item_site") else None)!s}, '
                f'auction_buyer_id={(self.auction_buyer_id if hasattr(self, "auction_buyer_id") else None)!s}, '
                f'auction_closing_date={(self.auction_closing_date if hasattr(self, "auction_closing_date") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.models.subscription_pricing_scheme import SubscriptionPricingScheme


class UpdatePricingScheme(object):

    """Implementation of the 'Update Pricing Scheme' model.

    The update pricing scheme request details.

    Attributes:
        billing_cycle_sequence (int): The billing cycle sequence.
        pricing_scheme (SubscriptionPricingScheme): The pricing scheme details.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "billing_cycle_sequence": 'billing_cycle_sequence',
        "pricing_scheme": 'pricing_scheme'
    }

    def __init__(self,
                 billing_cycle_sequence=None,
                 pricing_scheme=None):
        """Constructor for the UpdatePricingScheme class"""

        # Initialize members of the class
        self.billing_cycle_sequence = billing_cycle_sequence 
        self.pricing_scheme = pricing_scheme 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        billing_cycle_sequence = dictionary.get("billing_cycle_sequence") if dictionary.get("billing_cycle_sequence") else None
        pricing_scheme = SubscriptionPricingScheme.from_dictionary(dictionary.get('pricing_scheme')) if dictionary.get('pricing_scheme') else None
        # Return an object of this model
        return cls(billing_cycle_sequence,
                   pricing_scheme)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'billing_cycle_sequence={self.billing_cycle_sequence!r}, '
                f'pricing_scheme={self.pricing_scheme!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'billing_cycle_sequence={self.billing_cycle_sequence!s}, '
                f'pricing_scheme={self.pricing_scheme!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class UpcType(object):

    """Implementation of the 'UPC Type' enum.

    The Universal Product Code type.

    Attributes:
        UPC_A: The enum member of type str.
        UPC_B: The enum member of type str.
        UPC_C: The enum member of type str.
        UPC_D: The enum member of type str.
        UPC_E: The enum member of type str.
        UPC_2: The enum member of type str.
        UPC_5: The enum member of type str.

    """
    UPC_A = 'UPC-A'

    UPC_B = 'UPC-B'

    UPC_C = 'UPC-C'

    UPC_D = 'UPC-D'

    UPC_E = 'UPC-E'

    UPC_2 = 'UPC-2'

    UPC_5 = 'UPC-5'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class BalanceInformation(object):

    """Implementation of the 'Balance Information' model.

    The Balance information.

    Attributes:
        currency (str): The [three-character ISO-4217 currency
            code](/docs/integration/direct/rest/currency-codes/) that
            identifies the currency.
        primary (bool): Optional field representing if the currency is primary
            currency or not.
        total_balance (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        available_balance (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        withheld_balance (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "currency": 'currency',
        "total_balance": 'total_balance',
        "primary": 'primary',
        "available_balance": 'available_balance',
        "withheld_balance": 'withheld_balance'
    }

    _optionals = [
        'primary',
        'available_balance',
        'withheld_balance',
    ]

    def __init__(self,
                 currency=None,
                 total_balance=None,
                 primary=APIHelper.SKIP,
                 available_balance=APIHelper.SKIP,
                 withheld_balance=APIHelper.SKIP):
        """Constructor for the BalanceInformation class"""

        # Initialize members of the class
        self.currency = currency 
        if primary is not APIHelper.SKIP:
            self.primary = primary 
        self.total_balance = total_balance 
        if available_balance is not APIHelper.SKIP:
            self.available_balance = available_balance 
        if withheld_balance is not APIHelper.SKIP:
            self.withheld_balance = withheld_balance 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        currency = dictionary.get("currency") if dictionary.get("currency") else None
        total_balance = Money.from_dictionary(dictionary.get('total_balance')) if dictionary.get('total_balance') else None
        primary = dictionary.get("primary") if "primary" in dictionary.keys() else APIHelper.SKIP
        available_balance = Money.from_dictionary(dictionary.get('available_balance')) if 'available_balance' in dictionary.keys() else APIHelper.SKIP
        withheld_balance = Money.from_dictionary(dictionary.get('withheld_balance')) if 'withheld_balance' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(currency,
                   total_balance,
                   primary,
                   available_balance,
                   withheld_balance)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'currency={self.currency!r}, '
                f'primary={(self.primary if hasattr(self, "primary") else None)!r}, '
                f'total_balance={self.total_balance!r}, '
                f'available_balance={(self.available_balance if hasattr(self, "available_balance") else None)!r}, '
                f'withheld_balance={(self.withheld_balance if hasattr(self, "withheld_balance") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'currency={self.currency!s}, '
                f'primary={(self.primary if hasattr(self, "primary") else None)!s}, '
                f'total_balance={self.total_balance!s}, '
                f'available_balance={(self.available_balance if hasattr(self, "available_balance") else None)!s}, '
                f'withheld_balance={(self.withheld_balance if hasattr(self, "withheld_balance") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrdersCardVerificationMethod(object):

    """Implementation of the 'Orders Card Verification Method' enum.

    The method used for card verification.

    Attributes:
        SCA_ALWAYS: Selecting this option will attempt to force a strong
            customer authentication for the authorization/transaction. In
            countries where SCA has been defined and implemented it will
            result in a contingency and HATEOAS link being returned.  The API
            caller should redirect the payer to that link so that they can
            authenticate themselves against their issuing bank or other
            entity. As noted, the HATEOAS link is only available in all
            regions where strong authentication is supported, (e.g. in
            European countries where 3DS is live). Merchants can use this
            setting as an additional layer of security if they choose to. In
            all cases, when an authorization is requested the AVS/CVV results
            will be returned in the response.
        SCA_WHEN_REQUIRED: This is the default. When an authorization or
            transaction is attempted this option will return a contingency and
            HATEOAS link only when local regulations require strong customer
            authentication, (e.g. 3DS in countries and use cases where it is
            mandated). The API caller should redirect the payer to the link so
            that they can authenticate themselves. In all cases, when an
            authorization is requested the AVS/CVV results will be returned in
            the response.
        ENUM_3D_SECURE: The contingency surfaced as an additional security
            layer that helps prevent unauthorized card-not-present
            transactions and protects the merchant from exposure to fraud.
        AVS_CVV: Places a temporary hold on the card to ensure its validity.
            This process protects the merchant from exposure to fraud. This
            verification method will confirm that the address information or
            CVV included matches what the issuing bank has on file for the
            associated card, ensuring that only authorized card users are able
            to make purchases from you.

    """
    SCA_ALWAYS = 'SCA_ALWAYS'

    SCA_WHEN_REQUIRED = 'SCA_WHEN_REQUIRED'

    ENUM_3D_SECURE = '3D_SECURE'

    AVS_CVV = 'AVS_CVV'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class MobileWebContext(object):

    """Implementation of the 'Mobile Web Context' model.

    Buyer's mobile web browser context to app switch to the PayPal consumer
    app.

    Attributes:
        return_flow (MobileReturnFlow): Merchant preference on how the buyer
            can navigate back to merchant website post approving the
            transaction on the PayPal App.
        buyer_user_agent (str): User agent from the request originating from
            the buyer's device. This will be used to identify the buyer's
            operating system and browser versions. NOTE: Merchants must not
            alter or modify the buyer's device user agent.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "return_flow": 'return_flow',
        "buyer_user_agent": 'buyer_user_agent'
    }

    _optionals = [
        'return_flow',
        'buyer_user_agent',
    ]

    def __init__(self,
                 return_flow='AUTO',
                 buyer_user_agent=APIHelper.SKIP):
        """Constructor for the MobileWebContext class"""

        # Initialize members of the class
        self.return_flow = return_flow 
        if buyer_user_agent is not APIHelper.SKIP:
            self.buyer_user_agent = buyer_user_agent 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        return_flow = dictionary.get("return_flow") if dictionary.get("return_flow") else 'AUTO'
        buyer_user_agent = dictionary.get("buyer_user_agent") if dictionary.get("buyer_user_agent") else APIHelper.SKIP
        # Return an object of this model
        return cls(return_flow,
                   buyer_user_agent)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'return_flow={(self.return_flow if hasattr(self, "return_flow") else None)!r}, '
                f'buyer_user_agent={(self.buyer_user_agent if hasattr(self, "buyer_user_agent") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'return_flow={(self.return_flow if hasattr(self, "return_flow") else None)!s}, '
                f'buyer_user_agent={(self.buyer_user_agent if hasattr(self, "buyer_user_agent") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.billing_cycle import BillingCycle
from paypalserversdk.models.money import Money


class OrderBillingPlan(object):

    """Implementation of the 'Order Billing Plan' model.

    Metadata for merchant-managed recurring billing plans. Valid only during
    the saved payment method token or billing agreement creation.

    Attributes:
        billing_cycles (List[BillingCycle]): An array of billing cycles for
            trial billing and regular billing. A plan can have at most two
            trial cycles and only one regular cycle.
        setup_fee (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        name (str): Name of the recurring plan.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "billing_cycles": 'billing_cycles',
        "setup_fee": 'setup_fee',
        "name": 'name'
    }

    _optionals = [
        'setup_fee',
        'name',
    ]

    def __init__(self,
                 billing_cycles=None,
                 setup_fee=APIHelper.SKIP,
                 name=APIHelper.SKIP):
        """Constructor for the OrderBillingPlan class"""

        # Initialize members of the class
        self.billing_cycles = billing_cycles 
        if setup_fee is not APIHelper.SKIP:
            self.setup_fee = setup_fee 
        if name is not APIHelper.SKIP:
            self.name = name 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        billing_cycles = None
        if dictionary.get('billing_cycles') is not None:
            billing_cycles = [BillingCycle.from_dictionary(x) for x in dictionary.get('billing_cycles')]
        setup_fee = Money.from_dictionary(dictionary.get('setup_fee')) if 'setup_fee' in dictionary.keys() else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        # Return an object of this model
        return cls(billing_cycles,
                   setup_fee,
                   name)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'billing_cycles={self.billing_cycles!r}, '
                f'setup_fee={(self.setup_fee if hasattr(self, "setup_fee") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'billing_cycles={self.billing_cycles!s}, '
                f'setup_fee={(self.setup_fee if hasattr(self, "setup_fee") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CallbackConfiguration(object):

    """Implementation of the 'Callback Configuration' model.

    CallBack Configuration that the merchant can provide to PayPal/Venmo.

    Attributes:
        callback_events (List[CallbackEvents]): An array of callback events
            merchant can subscribe to for the corresponding callback url.
        callback_url (str): Merchant provided CallBack url.PayPal/Venmo will
            use this url to call the merchant back when the events occur
            .PayPal/Venmo expects a secured url usually in the https
            format.merchant can append the cart id or other params part of the
            url as query or path params.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "callback_events": 'callback_events',
        "callback_url": 'callback_url'
    }

    def __init__(self,
                 callback_events=None,
                 callback_url=None):
        """Constructor for the CallbackConfiguration class"""

        # Initialize members of the class
        self.callback_events = callback_events 
        self.callback_url = callback_url 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        callback_events = dictionary.get("callback_events") if dictionary.get("callback_events") else None
        callback_url = dictionary.get("callback_url") if dictionary.get("callback_url") else None
        # Return an object of this model
        return cls(callback_events,
                   callback_url)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'callback_events={self.callback_events!r}, '
                f'callback_url={self.callback_url!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'callback_events={self.callback_events!s}, '
                f'callback_url={self.callback_url!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaymentAdviceCode(object):

    """Implementation of the 'Payment Advice Code' enum.

    The declined payment transactions might have payment advice codes. The
    card networks, like Visa and Mastercard, return payment advice codes.

    Attributes:
        PAYMENTADVICE_01: For Mastercard, expired card account upgrade or
            portfolio sale conversion. Obtain new account information before
            next billing cycle.
        PAYMENTADVICE_02: For Mastercard, over credit limit or insufficient
            funds. Retry the transaction 72 hours later. For Visa, the card
            holder wants to stop only one specific payment in the recurring
            payment relationship. The merchant must NOT resubmit the same
            transaction. The merchant can continue the billing process in the
            subsequent billing period.
        PAYMENTADVICE_03: For Mastercard, account closed as fraudulent. Obtain
            another type of payment from customer due to account being closed
            or fraud. Possible reason: Account closed as fraudulent. For Visa,
            the card holder wants to stop all recurring payment transactions
            for a specific merchant. Stop recurring payment requests.
        PAYMENTADVICE_04: For Mastercard, token requirements not fulfilled for
            this token type.
        PAYMENTADVICE_21: For Mastercard, the card holder has been
            unsuccessful at canceling recurring payment through merchant. Stop
            recurring payment requests. For Visa, all recurring payments were
            canceled for the card number requested. Stop recurring payment
            requests.
        PAYMENTADVICE_22: For Mastercard, merchant does not qualify for
            product code.
        PAYMENTADVICE_24: For Mastercard, retry after 1 hour.
        PAYMENTADVICE_25: For Mastercard, retry after 24 hours.
        PAYMENTADVICE_26: For Mastercard, retry after 2 days.
        PAYMENTADVICE_27: For Mastercard, retry after 4 days.
        PAYMENTADVICE_28: For Mastercard, retry after 6 days.
        PAYMENTADVICE_29: For Mastercard, retry after 8 days.
        PAYMENTADVICE_30: For Mastercard, retry after 10 days .
        PAYMENTADVICE_40: For Mastercard, consumer non-reloadable prepaid card.
        PAYMENTADVICE_43: For Mastercard, consumer multi-use virtual card
            number.

    """
    PAYMENTADVICE_01 = '01'

    PAYMENTADVICE_02 = '02'

    PAYMENTADVICE_03 = '03'

    PAYMENTADVICE_04 = '04'

    PAYMENTADVICE_21 = '21'

    PAYMENTADVICE_22 = '22'

    PAYMENTADVICE_24 = '24'

    PAYMENTADVICE_25 = '25'

    PAYMENTADVICE_26 = '26'

    PAYMENTADVICE_27 = '27'

    PAYMENTADVICE_28 = '28'

    PAYMENTADVICE_29 = '29'

    PAYMENTADVICE_30 = '30'

    PAYMENTADVICE_40 = '40'

    PAYMENTADVICE_43 = '43'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class SimplePostalAddressCoarseGrained(object):

    """Implementation of the 'Simple Postal Address (Coarse-Grained)' model.

    A simple postal address with coarse-grained fields. Do not use for an
    international address. Use for backward compatibility only. Does not
    contain phone.

    Attributes:
        line_1 (str): The first line of the address. For example, number or
            street.
        line_2 (str): The second line of the address. For example, suite or
            apartment number.
        city (str): The city name.
        state (str): The [code](/docs/api/reference/state-codes/) for a US
            state or the equivalent for other countries. Required for
            transactions if the address is in one of these countries:
            [Argentina](/docs/api/reference/state-codes/#argentina),
            [Brazil](/docs/api/reference/state-codes/#brazil),
            [Canada](/docs/api/reference/state-codes/#canada),
            [China](/docs/api/reference/state-codes/#china),
            [India](/docs/api/reference/state-codes/#india),
            [Italy](/docs/api/reference/state-codes/#italy),
            [Japan](/docs/api/reference/state-codes/#japan),
            [Mexico](/docs/api/reference/state-codes/#mexico),
            [Thailand](/docs/api/reference/state-codes/#thailand), or [United
            States](/docs/api/reference/state-codes/#usa). Maximum length is
            40 single-byte characters.
        country_code (str): The [two-character ISO 3166-1
            code](/docs/integration/direct/rest/country-codes/) that
            identifies the country or region. Note: The country code for Great
            Britain is GB and not UK as used in the top-level domain names for
            that country. Use the `C2` country code for China worldwide for
            comparable uncontrolled price (CUP) method, bank card, and
            cross-border transactions.
        postal_code (str): The postal code, which is the zip code or
            equivalent. Typically required for countries with a postal code or
            an equivalent. See [postal
            code](https://en.wikipedia.org/wiki/Postal_code).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "line_1": 'line1',
        "city": 'city',
        "country_code": 'country_code',
        "line_2": 'line2',
        "state": 'state',
        "postal_code": 'postal_code'
    }

    _optionals = [
        'line_2',
        'state',
        'postal_code',
    ]

    def __init__(self,
                 line_1=None,
                 city=None,
                 country_code=None,
                 line_2=APIHelper.SKIP,
                 state=APIHelper.SKIP,
                 postal_code=APIHelper.SKIP):
        """Constructor for the SimplePostalAddressCoarseGrained class"""

        # Initialize members of the class
        self.line_1 = line_1 
        if line_2 is not APIHelper.SKIP:
            self.line_2 = line_2 
        self.city = city 
        if state is not APIHelper.SKIP:
            self.state = state 
        self.country_code = country_code 
        if postal_code is not APIHelper.SKIP:
            self.postal_code = postal_code 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        line_1 = dictionary.get("line1") if dictionary.get("line1") else None
        city = dictionary.get("city") if dictionary.get("city") else None
        country_code = dictionary.get("country_code") if dictionary.get("country_code") else None
        line_2 = dictionary.get("line2") if dictionary.get("line2") else APIHelper.SKIP
        state = dictionary.get("state") if dictionary.get("state") else APIHelper.SKIP
        postal_code = dictionary.get("postal_code") if dictionary.get("postal_code") else APIHelper.SKIP
        # Return an object of this model
        return cls(line_1,
                   city,
                   country_code,
                   line_2,
                   state,
                   postal_code)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'line_1={self.line_1!r}, '
                f'line_2={(self.line_2 if hasattr(self, "line_2") else None)!r}, '
                f'city={self.city!r}, '
                f'state={(self.state if hasattr(self, "state") else None)!r}, '
                f'country_code={self.country_code!r}, '
                f'postal_code={(self.postal_code if hasattr(self, "postal_code") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'line_1={self.line_1!s}, '
                f'line_2={(self.line_2 if hasattr(self, "line_2") else None)!s}, '
                f'city={self.city!s}, '
                f'state={(self.state if hasattr(self, "state") else None)!s}, '
                f'country_code={self.country_code!s}, '
                f'postal_code={(self.postal_code if hasattr(self, "postal_code") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.money import Money
from paypalserversdk.models.plan_details import PlanDetails
from paypalserversdk.models.subscriber import Subscriber
from paypalserversdk.models.subscription_billing_information import SubscriptionBillingInformation


class Subscription(object):

    """Implementation of the 'Subscription' model.

    The subscription details.

    Attributes:
        id (str): The PayPal-generated ID for the subscription.
        plan_id (str): The ID of the plan.
        start_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        quantity (str): The quantity of the product in the subscription.
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        subscriber (Subscriber): The subscriber response information.
        billing_info (SubscriptionBillingInformation): The billing details for
            the subscription. If the subscription was or is active, these
            fields are populated.
        create_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        update_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        custom_id (str): The custom id for the subscription. Can be invoice id.
        plan_overridden (bool): Indicates whether the subscription has
            overridden any plan attributes.
        plan (PlanDetails): The plan details.
        links (List[LinkDescription]): An array of request-related [HATEOAS
            links](/docs/api/reference/api-responses/#hateoas-links).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "plan_id": 'plan_id',
        "start_time": 'start_time',
        "quantity": 'quantity',
        "shipping_amount": 'shipping_amount',
        "subscriber": 'subscriber',
        "billing_info": 'billing_info',
        "create_time": 'create_time',
        "update_time": 'update_time',
        "custom_id": 'custom_id',
        "plan_overridden": 'plan_overridden',
        "plan": 'plan',
        "links": 'links'
    }

    _optionals = [
        'id',
        'plan_id',
        'start_time',
        'quantity',
        'shipping_amount',
        'subscriber',
        'billing_info',
        'create_time',
        'update_time',
        'custom_id',
        'plan_overridden',
        'plan',
        'links',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 plan_id=APIHelper.SKIP,
                 start_time=APIHelper.SKIP,
                 quantity=APIHelper.SKIP,
                 shipping_amount=APIHelper.SKIP,
                 subscriber=APIHelper.SKIP,
                 billing_info=APIHelper.SKIP,
                 create_time=APIHelper.SKIP,
                 update_time=APIHelper.SKIP,
                 custom_id=APIHelper.SKIP,
                 plan_overridden=APIHelper.SKIP,
                 plan=APIHelper.SKIP,
                 links=APIHelper.SKIP):
        """Constructor for the Subscription class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if plan_id is not APIHelper.SKIP:
            self.plan_id = plan_id 
        if start_time is not APIHelper.SKIP:
            self.start_time = start_time 
        if quantity is not APIHelper.SKIP:
            self.quantity = quantity 
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if subscriber is not APIHelper.SKIP:
            self.subscriber = subscriber 
        if billing_info is not APIHelper.SKIP:
            self.billing_info = billing_info 
        if create_time is not APIHelper.SKIP:
            self.create_time = create_time 
        if update_time is not APIHelper.SKIP:
            self.update_time = update_time 
        if custom_id is not APIHelper.SKIP:
            self.custom_id = custom_id 
        if plan_overridden is not APIHelper.SKIP:
            self.plan_overridden = plan_overridden 
        if plan is not APIHelper.SKIP:
            self.plan = plan 
        if links is not APIHelper.SKIP:
            self.links = links 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        plan_id = dictionary.get("plan_id") if dictionary.get("plan_id") else APIHelper.SKIP
        start_time = dictionary.get("start_time") if dictionary.get("start_time") else APIHelper.SKIP
        quantity = dictionary.get("quantity") if dictionary.get("quantity") else APIHelper.SKIP
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        subscriber = Subscriber.from_dictionary(dictionary.get('subscriber')) if 'subscriber' in dictionary.keys() else APIHelper.SKIP
        billing_info = SubscriptionBillingInformation.from_dictionary(dictionary.get('billing_info')) if 'billing_info' in dictionary.keys() else APIHelper.SKIP
        create_time = dictionary.get("create_time") if dictionary.get("create_time") else APIHelper.SKIP
        update_time = dictionary.get("update_time") if dictionary.get("update_time") else APIHelper.SKIP
        custom_id = dictionary.get("custom_id") if dictionary.get("custom_id") else APIHelper.SKIP
        plan_overridden = dictionary.get("plan_overridden") if "plan_overridden" in dictionary.keys() else APIHelper.SKIP
        plan = PlanDetails.from_dictionary(dictionary.get('plan')) if 'plan' in dictionary.keys() else APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   plan_id,
                   start_time,
                   quantity,
                   shipping_amount,
                   subscriber,
                   billing_info,
                   create_time,
                   update_time,
                   custom_id,
                   plan_overridden,
                   plan,
                   links)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'plan_id={(self.plan_id if hasattr(self, "plan_id") else None)!r}, '
                f'start_time={(self.start_time if hasattr(self, "start_time") else None)!r}, '
                f'quantity={(self.quantity if hasattr(self, "quantity") else None)!r}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!r}, '
                f'subscriber={(self.subscriber if hasattr(self, "subscriber") else None)!r}, '
                f'billing_info={(self.billing_info if hasattr(self, "billing_info") else None)!r}, '
                f'create_time={(self.create_time if hasattr(self, "create_time") else None)!r}, '
                f'update_time={(self.update_time if hasattr(self, "update_time") else None)!r}, '
                f'custom_id={(self.custom_id if hasattr(self, "custom_id") else None)!r}, '
                f'plan_overridden={(self.plan_overridden if hasattr(self, "plan_overridden") else None)!r}, '
                f'plan={(self.plan if hasattr(self, "plan") else None)!r}, '
                f'links={(self.links if hasattr(self, "links") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'plan_id={(self.plan_id if hasattr(self, "plan_id") else None)!s}, '
                f'start_time={(self.start_time if hasattr(self, "start_time") else None)!s}, '
                f'quantity={(self.quantity if hasattr(self, "quantity") else None)!s}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!s}, '
                f'subscriber={(self.subscriber if hasattr(self, "subscriber") else None)!s}, '
                f'billing_info={(self.billing_info if hasattr(self, "billing_info") else None)!s}, '
                f'create_time={(self.create_time if hasattr(self, "create_time") else None)!s}, '
                f'update_time={(self.update_time if hasattr(self, "update_time") else None)!s}, '
                f'custom_id={(self.custom_id if hasattr(self, "custom_id") else None)!s}, '
                f'plan_overridden={(self.plan_overridden if hasattr(self, "plan_overridden") else None)!s}, '
                f'plan={(self.plan if hasattr(self, "plan") else None)!s}, '
                f'links={(self.links if hasattr(self, "links") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VaultUserAction(object):

    """Implementation of the 'Vault User Action' enum.

    User Action on action to be performed after a successful payer approval.

    Attributes:
        SETUP_NOW: After you redirect the customer to the PayPal payment page,
            a Setup Now button appears. Use this option when no additional
            inputs are needed from merchant site and to create the billing
            agreement immediately when the customer clicks Setup Now.
        CONTINUE: After you redirect the customer to the PayPal payment page,
            a Continue button appears. Use this option when you want to
            redirect the customer from the completed payment page to the
            merchant site for additional inputs without immediately creating
            the billing agreement.

    """
    SETUP_NOW = 'SETUP_NOW'

    CONTINUE = 'CONTINUE'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class VenmoPaymentTokenUsagePattern(object):

    """Implementation of the 'Venmo Payment Token Usage Pattern' enum.

    Expected business/pricing model for the billing agreement.

    Attributes:
        IMMEDIATE: On-demand instant payments – non-recurring, pre-paid,
            variable amount, variable frequency.
        DEFERRED: Pay after use, non-recurring post-paid, variable amount,
            irregular frequency.
        RECURRING_PREPAID: Pay upfront fixed or variable amount on a fixed
            date before the goods/service is delivered.
        RECURRING_POSTPAID: Pay on a fixed date based on usage or consumption
            after the goods/service is delivered.
        THRESHOLD_PREPAID: Charge payer when the set amount is reached or
            monthly billing cycle, whichever comes first, before the
            goods/service is delivered.
        THRESHOLD_POSTPAID: Charge payer when the set amount is reached or
            monthly billing cycle, whichever comes first, after the
            goods/service is delivered.

    """
    IMMEDIATE = 'IMMEDIATE'

    DEFERRED = 'DEFERRED'

    RECURRING_PREPAID = 'RECURRING_PREPAID'

    RECURRING_POSTPAID = 'RECURRING_POSTPAID'

    THRESHOLD_PREPAID = 'THRESHOLD_PREPAID'

    THRESHOLD_POSTPAID = 'THRESHOLD_POSTPAID'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrderStatus(object):

    """Implementation of the 'Order Status' enum.

    The order status.

    Attributes:
        CREATED: The order was created with the specified context.
        SAVED: The order was saved and persisted. The order status continues
            to be in progress until a capture is made with final_capture =
            true for all purchase units within the order.
        APPROVED: The customer approved the payment through the PayPal wallet
            or another form of guest or unbranded payment. For example, a
            card, bank account, or so on.
        VOIDED: All purchase units in the order are voided.
        COMPLETED: The intent of the order was completed and a `payments`
            resource was created. Important: Check the payment status in
            `purchase_units[].payments.captures[].status` before fulfilling
            the order. A completed order can indicate a payment was
            authorized, an authorized payment was captured, or a payment was
            declined.
        PAYER_ACTION_REQUIRED: The order requires an action from the payer
            (e.g. 3DS authentication). Redirect the payer to the
            "rel":"payer-action" HATEOAS link returned as part of the response
            prior to authorizing or capturing the order. Some payment sources
            may not return a payer-action HATEOAS link (eg. MB WAY). For these
            payment sources the payer-action is managed by the scheme itself
            (eg. through SMS, email, in-app notification, etc).

    """
    CREATED = 'CREATED'

    SAVED = 'SAVED'

    APPROVED = 'APPROVED'

    VOIDED = 'VOIDED'

    COMPLETED = 'COMPLETED'

    PAYER_ACTION_REQUIRED = 'PAYER_ACTION_REQUIRED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class IntervalUnit(object):

    """Implementation of the 'Interval Unit' enum.

    The interval at which the subscription is charged or billed.

    Attributes:
        DAY: A daily billing cycle.
        WEEK: A weekly billing cycle.
        MONTH: A monthly billing cycle.
        YEAR: A yearly billing cycle.

    """
    DAY = 'DAY'

    WEEK = 'WEEK'

    MONTH = 'MONTH'

    YEAR = 'YEAR'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.stored_payment_source import StoredPaymentSource


class OrderConfirmApplicationContext(object):

    """Implementation of the 'Order Confirm Application Context' model.

    Customizes the payer confirmation experience.

    Attributes:
        brand_name (str): Label to present to your payer as part of the PayPal
            hosted web experience.
        locale (str): The [language
            tag](https://tools.ietf.org/html/bcp47#section-2) for the language
            in which to localize the error-related strings, such as messages,
            issues, and suggested actions. The tag is made up of the [ISO
            639-2 language
            code](https://www.loc.gov/standards/iso639-2/php/code_list.php),
            the optional [ISO-15924 script
            tag](https://www.unicode.org/iso15924/codelists.html), and the
            [ISO-3166 alpha-2 country
            code](/api/rest/reference/country-codes/) or [M49 region
            code](https://unstats.un.org/unsd/methodology/m49/).
        return_url (str): The URL where the customer is redirected after the
            customer approves the payment.
        cancel_url (str): The URL where the customer is redirected after the
            customer cancels the payment.
        stored_payment_source (StoredPaymentSource): Provides additional
            details to process a payment using a `payment_source` that has
            been stored or is intended to be stored (also referred to as
            stored_credential or card-on-file). Parameter compatibility:
            `payment_type=ONE_TIME` is compatible only with
            `payment_initiator=CUSTOMER`. `usage=FIRST` is compatible only
            with `payment_initiator=CUSTOMER`.
            `previous_transaction_reference` or
            `previous_network_transaction_reference` is compatible only with
            `payment_initiator=MERCHANT`. Only one of the parameters -
            `previous_transaction_reference` and
            `previous_network_transaction_reference` - can be present in the
            request.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "brand_name": 'brand_name',
        "locale": 'locale',
        "return_url": 'return_url',
        "cancel_url": 'cancel_url',
        "stored_payment_source": 'stored_payment_source'
    }

    _optionals = [
        'brand_name',
        'locale',
        'return_url',
        'cancel_url',
        'stored_payment_source',
    ]

    def __init__(self,
                 brand_name=APIHelper.SKIP,
                 locale=APIHelper.SKIP,
                 return_url=APIHelper.SKIP,
                 cancel_url=APIHelper.SKIP,
                 stored_payment_source=APIHelper.SKIP):
        """Constructor for the OrderConfirmApplicationContext class"""

        # Initialize members of the class
        if brand_name is not APIHelper.SKIP:
            self.brand_name = brand_name 
        if locale is not APIHelper.SKIP:
            self.locale = locale 
        if return_url is not APIHelper.SKIP:
            self.return_url = return_url 
        if cancel_url is not APIHelper.SKIP:
            self.cancel_url = cancel_url 
        if stored_payment_source is not APIHelper.SKIP:
            self.stored_payment_source = stored_payment_source 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        brand_name = dictionary.get("brand_name") if dictionary.get("brand_name") else APIHelper.SKIP
        locale = dictionary.get("locale") if dictionary.get("locale") else APIHelper.SKIP
        return_url = dictionary.get("return_url") if dictionary.get("return_url") else APIHelper.SKIP
        cancel_url = dictionary.get("cancel_url") if dictionary.get("cancel_url") else APIHelper.SKIP
        stored_payment_source = StoredPaymentSource.from_dictionary(dictionary.get('stored_payment_source')) if 'stored_payment_source' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(brand_name,
                   locale,
                   return_url,
                   cancel_url,
                   stored_payment_source)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!r}, '
                f'locale={(self.locale if hasattr(self, "locale") else None)!r}, '
                f'return_url={(self.return_url if hasattr(self, "return_url") else None)!r}, '
                f'cancel_url={(self.cancel_url if hasattr(self, "cancel_url") else None)!r}, '
                f'stored_payment_source={(self.stored_payment_source if hasattr(self, "stored_payment_source") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!s}, '
                f'locale={(self.locale if hasattr(self, "locale") else None)!s}, '
                f'return_url={(self.return_url if hasattr(self, "return_url") else None)!s}, '
                f'cancel_url={(self.cancel_url if hasattr(self, "cancel_url") else None)!s}, '
                f'stored_payment_source={(self.stored_payment_source if hasattr(self, "stored_payment_source") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ApplicationContextUserAction(object):

    """Implementation of the 'Application Context User Action' enum.

    Configures the label name to `Continue` or `Subscribe Now` for
    subscription consent experience.

    Attributes:
        CONTINUE: After you redirect the customer to the PayPal subscription
            consent page, a Continue button appears. Use this option when you
            want to control the activation of the subscription and do not want
            PayPal to activate the subscription.
        SUBSCRIBE_NOW: After you redirect the customer to the PayPal
            subscription consent page, a Subscribe Now button appears. Use
            this option when you want PayPal to activate the subscription.

    """
    CONTINUE = 'CONTINUE'

    SUBSCRIBE_NOW = 'SUBSCRIBE_NOW'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.amount_with_breakdown import AmountWithBreakdown
from paypalserversdk.models.item_request import ItemRequest
from paypalserversdk.models.payee_base import PayeeBase
from paypalserversdk.models.payment_instruction import PaymentInstruction
from paypalserversdk.models.shipping_details import ShippingDetails
from paypalserversdk.models.supplementary_data import SupplementaryData


class PurchaseUnitRequest(object):

    """Implementation of the 'Purchase Unit Request' model.

    The purchase unit request. Includes required information for the payment
    contract.

    Attributes:
        reference_id (str): The API caller-provided external ID for the
            purchase unit. Required for multiple purchase units when you must
            update the order through `PATCH`. If you omit this value and the
            order contains only one purchase unit, PayPal sets this value to
            `default`.
        amount (AmountWithBreakdown): The total order amount with an optional
            breakdown that provides details, such as the total item amount,
            total tax amount, shipping, handling, insurance, and discounts, if
            any. If you specify `amount.breakdown`, the amount equals
            `item_total` plus `tax_total` plus `shipping` plus `handling` plus
            `insurance` minus `shipping_discount` minus discount. The amount
            must be a positive number. For listed of supported currencies and
            decimal precision, see the PayPal REST APIs Currency Codes.
        payee (PayeeBase): The merchant who receives the funds and fulfills
            the order. The merchant is also known as the payee.
        payment_instruction (PaymentInstruction): Any additional payment
            instructions to be consider during payment processing. This
            processing instruction is applicable for Capturing an order or
            Authorizing an Order.
        description (str): This field supports up to 3,000 characters, but any
            content beyond 127 characters (including spaces) will be
            truncated. The 127 character limit is reflected in the response
            representation of this field. The purchase description. The
            maximum length of the character is dependent on the type of
            characters used. The character length is specified assuming a US
            ASCII character. Depending on type of character; (e.g. accented
            character, Japanese characters) the number of characters that that
            can be specified as input might not equal the permissible max
            length.
        custom_id (str): The API caller-provided external ID. Used to
            reconcile client transactions with PayPal transactions. Appears in
            transaction and settlement reports but is not visible to the payer.
        invoice_id (str): The API caller-provided external invoice number for
            this order. Appears in both the payer's transaction history and
            the emails that the payer receives. invoice_id values are required
            to be unique within each merchant account by default. Although the
            uniqueness validation is configurable, disabling this behavior
            will remove the account's ability to use invoice_id in other APIs
            as an identifier. It is highly recommended to keep a unique
            invoice_id for each Order.
        soft_descriptor (str): This field supports up to 127 characters, but
            any content beyond 22 characters (including spaces) will be
            truncated. The 22 character limit is reflected in the response
            representation of this field. The soft descriptor is the dynamic
            text used to construct the statement descriptor that appears on a
            payer's card statement. If an Order is paid using the "PayPal
            Wallet", the statement descriptor will appear in following format
            on the payer's card statement:
            PAYPAL_prefix+(space)+merchant_descriptor+(space)+ soft_descriptor
            Note: The merchant descriptor is the descriptor of the merchant’s
            payment receiving preferences which can be seen by logging into
            the merchant account
            https://www.sandbox.paypal.com/businessprofile/settings/info/edit
            The PAYPAL prefix uses 8 characters. Only the first 22 characters
            will be displayed in the statement. For example, if: The PayPal
            prefix toggle is PAYPAL *. The merchant descriptor in the profile
            is Janes Gift. The soft descriptor is 800-123-1234. Then, the
            statement descriptor on the card is PAYPAL * Janes Gift 80.
        items (List[ItemRequest]): An array of items that the customer
            purchases from the merchant.
        shipping (ShippingDetails): The shipping details.
        supplementary_data (SupplementaryData): Supplementary data about a
            payment. This object passes information that can be used to
            improve risk assessments and processing costs, for example, by
            providing Level 2 and Level 3 payment data.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount": 'amount',
        "reference_id": 'reference_id',
        "payee": 'payee',
        "payment_instruction": 'payment_instruction',
        "description": 'description',
        "custom_id": 'custom_id',
        "invoice_id": 'invoice_id',
        "soft_descriptor": 'soft_descriptor',
        "items": 'items',
        "shipping": 'shipping',
        "supplementary_data": 'supplementary_data'
    }

    _optionals = [
        'reference_id',
        'payee',
        'payment_instruction',
        'description',
        'custom_id',
        'invoice_id',
        'soft_descriptor',
        'items',
        'shipping',
        'supplementary_data',
    ]

    def __init__(self,
                 amount=None,
                 reference_id=APIHelper.SKIP,
                 payee=APIHelper.SKIP,
                 payment_instruction=APIHelper.SKIP,
                 description=APIHelper.SKIP,
                 custom_id=APIHelper.SKIP,
                 invoice_id=APIHelper.SKIP,
                 soft_descriptor=APIHelper.SKIP,
                 items=APIHelper.SKIP,
                 shipping=APIHelper.SKIP,
                 supplementary_data=APIHelper.SKIP):
        """Constructor for the PurchaseUnitRequest class"""

        # Initialize members of the class
        if reference_id is not APIHelper.SKIP:
            self.reference_id = reference_id 
        self.amount = amount 
        if payee is not APIHelper.SKIP:
            self.payee = payee 
        if payment_instruction is not APIHelper.SKIP:
            self.payment_instruction = payment_instruction 
        if description is not APIHelper.SKIP:
            self.description = description 
        if custom_id is not APIHelper.SKIP:
            self.custom_id = custom_id 
        if invoice_id is not APIHelper.SKIP:
            self.invoice_id = invoice_id 
        if soft_descriptor is not APIHelper.SKIP:
            self.soft_descriptor = soft_descriptor 
        if items is not APIHelper.SKIP:
            self.items = items 
        if shipping is not APIHelper.SKIP:
            self.shipping = shipping 
        if supplementary_data is not APIHelper.SKIP:
            self.supplementary_data = supplementary_data 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = AmountWithBreakdown.from_dictionary(dictionary.get('amount')) if dictionary.get('amount') else None
        reference_id = dictionary.get("reference_id") if dictionary.get("reference_id") else APIHelper.SKIP
        payee = PayeeBase.from_dictionary(dictionary.get('payee')) if 'payee' in dictionary.keys() else APIHelper.SKIP
        payment_instruction = PaymentInstruction.from_dictionary(dictionary.get('payment_instruction')) if 'payment_instruction' in dictionary.keys() else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        custom_id = dictionary.get("custom_id") if dictionary.get("custom_id") else APIHelper.SKIP
        invoice_id = dictionary.get("invoice_id") if dictionary.get("invoice_id") else APIHelper.SKIP
        soft_descriptor = dictionary.get("soft_descriptor") if dictionary.get("soft_descriptor") else APIHelper.SKIP
        items = None
        if dictionary.get('items') is not None:
            items = [ItemRequest.from_dictionary(x) for x in dictionary.get('items')]
        else:
            items = APIHelper.SKIP
        shipping = ShippingDetails.from_dictionary(dictionary.get('shipping')) if 'shipping' in dictionary.keys() else APIHelper.SKIP
        supplementary_data = SupplementaryData.from_dictionary(dictionary.get('supplementary_data')) if 'supplementary_data' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(amount,
                   reference_id,
                   payee,
                   payment_instruction,
                   description,
                   custom_id,
                   invoice_id,
                   soft_descriptor,
                   items,
                   shipping,
                   supplementary_data)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'reference_id={(self.reference_id if hasattr(self, "reference_id") else None)!r}, '
                f'amount={self.amount!r}, '
                f'payee={(self.payee if hasattr(self, "payee") else None)!r}, '
                f'payment_instruction={(self.payment_instruction if hasattr(self, "payment_instruction") else None)!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r}, '
                f'custom_id={(self.custom_id if hasattr(self, "custom_id") else None)!r}, '
                f'invoice_id={(self.invoice_id if hasattr(self, "invoice_id") else None)!r}, '
                f'soft_descriptor={(self.soft_descriptor if hasattr(self, "soft_descriptor") else None)!r}, '
                f'items={(self.items if hasattr(self, "items") else None)!r}, '
                f'shipping={(self.shipping if hasattr(self, "shipping") else None)!r}, '
                f'supplementary_data={(self.supplementary_data if hasattr(self, "supplementary_data") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'reference_id={(self.reference_id if hasattr(self, "reference_id") else None)!s}, '
                f'amount={self.amount!s}, '
                f'payee={(self.payee if hasattr(self, "payee") else None)!s}, '
                f'payment_instruction={(self.payment_instruction if hasattr(self, "payment_instruction") else None)!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s}, '
                f'custom_id={(self.custom_id if hasattr(self, "custom_id") else None)!s}, '
                f'invoice_id={(self.invoice_id if hasattr(self, "invoice_id") else None)!s}, '
                f'soft_descriptor={(self.soft_descriptor if hasattr(self, "soft_descriptor") else None)!s}, '
                f'items={(self.items if hasattr(self, "items") else None)!s}, '
                f'shipping={(self.shipping if hasattr(self, "shipping") else None)!s}, '
                f'supplementary_data={(self.supplementary_data if hasattr(self, "supplementary_data") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrderTrackerStatus(object):

    """Implementation of the 'Order Tracker Status' enum.

    The status of the item shipment.

    Attributes:
        CANCELLED: The shipment was cancelled and the tracking number no
            longer applies.
        SHIPPED: The merchant has assigned a tracking number to the items
            being shipped from the Order. This does not correspond to the
            carrier's actual status for the shipment. The latest status of the
            parcel must be retrieved from the carrier.

    """
    CANCELLED = 'CANCELLED'

    SHIPPED = 'SHIPPED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.cycle_execution import CycleExecution
from paypalserversdk.models.failed_payment_details import FailedPaymentDetails
from paypalserversdk.models.last_payment_details import LastPaymentDetails
from paypalserversdk.models.money import Money


class SubscriptionBillingInformation(object):

    """Implementation of the 'Subscription Billing Information' model.

    The billing details for the subscription. If the subscription was or is
    active, these fields are populated.

    Attributes:
        outstanding_balance (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        cycle_executions (List[CycleExecution]): The trial and regular billing
            executions.
        last_payment (LastPaymentDetails): The details for the last payment.
        next_billing_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        final_payment_time (str): The date and time, in [Internet date and
            time format](https://tools.ietf.org/html/rfc3339#section-5.6).
            Seconds are required while fractional seconds are optional. Note:
            The regular expression provides guidance but does not reject all
            invalid dates.
        failed_payments_count (int): The number of consecutive payment
            failures. Resets to `0` after a successful payment. If this
            reaches the `payment_failure_threshold` value, the subscription
            updates to the `SUSPENDED` state.
        last_failed_payment (FailedPaymentDetails): The details for the failed
            payment of the subscription.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "outstanding_balance": 'outstanding_balance',
        "failed_payments_count": 'failed_payments_count',
        "cycle_executions": 'cycle_executions',
        "last_payment": 'last_payment',
        "next_billing_time": 'next_billing_time',
        "final_payment_time": 'final_payment_time',
        "last_failed_payment": 'last_failed_payment'
    }

    _optionals = [
        'cycle_executions',
        'last_payment',
        'next_billing_time',
        'final_payment_time',
        'last_failed_payment',
    ]

    def __init__(self,
                 outstanding_balance=None,
                 failed_payments_count=None,
                 cycle_executions=APIHelper.SKIP,
                 last_payment=APIHelper.SKIP,
                 next_billing_time=APIHelper.SKIP,
                 final_payment_time=APIHelper.SKIP,
                 last_failed_payment=APIHelper.SKIP):
        """Constructor for the SubscriptionBillingInformation class"""

        # Initialize members of the class
        self.outstanding_balance = outstanding_balance 
        if cycle_executions is not APIHelper.SKIP:
            self.cycle_executions = cycle_executions 
        if last_payment is not APIHelper.SKIP:
            self.last_payment = last_payment 
        if next_billing_time is not APIHelper.SKIP:
            self.next_billing_time = next_billing_time 
        if final_payment_time is not APIHelper.SKIP:
            self.final_payment_time = final_payment_time 
        self.failed_payments_count = failed_payments_count 
        if last_failed_payment is not APIHelper.SKIP:
            self.last_failed_payment = last_failed_payment 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        outstanding_balance = Money.from_dictionary(dictionary.get('outstanding_balance')) if dictionary.get('outstanding_balance') else None
        failed_payments_count = dictionary.get("failed_payments_count") if dictionary.get("failed_payments_count") else None
        cycle_executions = None
        if dictionary.get('cycle_executions') is not None:
            cycle_executions = [CycleExecution.from_dictionary(x) for x in dictionary.get('cycle_executions')]
        else:
            cycle_executions = APIHelper.SKIP
        last_payment = LastPaymentDetails.from_dictionary(dictionary.get('last_payment')) if 'last_payment' in dictionary.keys() else APIHelper.SKIP
        next_billing_time = dictionary.get("next_billing_time") if dictionary.get("next_billing_time") else APIHelper.SKIP
        final_payment_time = dictionary.get("final_payment_time") if dictionary.get("final_payment_time") else APIHelper.SKIP
        last_failed_payment = FailedPaymentDetails.from_dictionary(dictionary.get('last_failed_payment')) if 'last_failed_payment' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(outstanding_balance,
                   failed_payments_count,
                   cycle_executions,
                   last_payment,
                   next_billing_time,
                   final_payment_time,
                   last_failed_payment)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'outstanding_balance={self.outstanding_balance!r}, '
                f'cycle_executions={(self.cycle_executions if hasattr(self, "cycle_executions") else None)!r}, '
                f'last_payment={(self.last_payment if hasattr(self, "last_payment") else None)!r}, '
                f'next_billing_time={(self.next_billing_time if hasattr(self, "next_billing_time") else None)!r}, '
                f'final_payment_time={(self.final_payment_time if hasattr(self, "final_payment_time") else None)!r}, '
                f'failed_payments_count={self.failed_payments_count!r}, '
                f'last_failed_payment={(self.last_failed_payment if hasattr(self, "last_failed_payment") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'outstanding_balance={self.outstanding_balance!s}, '
                f'cycle_executions={(self.cycle_executions if hasattr(self, "cycle_executions") else None)!s}, '
                f'last_payment={(self.last_payment if hasattr(self, "last_payment") else None)!s}, '
                f'next_billing_time={(self.next_billing_time if hasattr(self, "next_billing_time") else None)!s}, '
                f'final_payment_time={(self.final_payment_time if hasattr(self, "final_payment_time") else None)!s}, '
                f'failed_payments_count={self.failed_payments_count!s}, '
                f'last_failed_payment={(self.last_failed_payment if hasattr(self, "last_failed_payment") else None)!s})')




"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.http.configurations.proxy_settings import ProxySettings as CoreProxySettings

class ProxySettings(CoreProxySettings):
    """
    A simple data model for configuring HTTP(S) proxy settings.
    """



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.customer_information import CustomerInformation
from paypalserversdk.models.vault_instruction import VaultInstruction


class ApplePayAttributes(object):

    """Implementation of the 'Apple Pay Attributes' model.

    Additional attributes associated with apple pay.

    Attributes:
        customer (CustomerInformation): This object represents a merchant’s
            customer, allowing them to store contact details, and track all
            payments associated with the same customer.
        vault (VaultInstruction): Base vaulting specification. The object can
            be extended for specific use cases within each payment_source that
            supports vaulting.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "customer": 'customer',
        "vault": 'vault'
    }

    _optionals = [
        'customer',
        'vault',
    ]

    def __init__(self,
                 customer=APIHelper.SKIP,
                 vault=APIHelper.SKIP):
        """Constructor for the ApplePayAttributes class"""

        # Initialize members of the class
        if customer is not APIHelper.SKIP:
            self.customer = customer 
        if vault is not APIHelper.SKIP:
            self.vault = vault 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        customer = CustomerInformation.from_dictionary(dictionary.get('customer')) if 'customer' in dictionary.keys() else APIHelper.SKIP
        vault = VaultInstruction.from_dictionary(dictionary.get('vault')) if 'vault' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(customer,
                   vault)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'customer={(self.customer if hasattr(self, "customer") else None)!r}, '
                f'vault={(self.vault if hasattr(self, "vault") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'customer={(self.customer if hasattr(self, "customer") else None)!s}, '
                f'vault={(self.vault if hasattr(self, "vault") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class SubscriptionPlanStatus(object):

    """Implementation of the 'Subscription Plan Status' enum.

    The plan status.

    Attributes:
        CREATED: The plan was created. You cannot create subscriptions for a
            plan in this state.
        INACTIVE: The plan is inactive.
        ACTIVE: The plan is active. You can only create subscriptions for a
            plan in this state.

    """
    CREATED = 'CREATED'

    INACTIVE = 'INACTIVE'

    ACTIVE = 'ACTIVE'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.name import Name
from paypalserversdk.models.shipping_details import ShippingDetails
from paypalserversdk.models.subscription_payment_source_response import SubscriptionPaymentSourceResponse


class Subscriber(object):

    """Implementation of the 'Subscriber' model.

    The subscriber response information.

    Attributes:
        name (Name): The name of the party.
        shipping_address (ShippingDetails): The shipping details.
        payment_source (SubscriptionPaymentSourceResponse): The payment source
            used to fund the payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "shipping_address": 'shipping_address',
        "payment_source": 'payment_source'
    }

    _optionals = [
        'name',
        'shipping_address',
        'payment_source',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 shipping_address=APIHelper.SKIP,
                 payment_source=APIHelper.SKIP):
        """Constructor for the Subscriber class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if shipping_address is not APIHelper.SKIP:
            self.shipping_address = shipping_address 
        if payment_source is not APIHelper.SKIP:
            self.payment_source = payment_source 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = Name.from_dictionary(dictionary.get('name')) if 'name' in dictionary.keys() else APIHelper.SKIP
        shipping_address = ShippingDetails.from_dictionary(dictionary.get('shipping_address')) if 'shipping_address' in dictionary.keys() else APIHelper.SKIP
        payment_source = SubscriptionPaymentSourceResponse.from_dictionary(dictionary.get('payment_source')) if 'payment_source' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   shipping_address,
                   payment_source)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'shipping_address={(self.shipping_address if hasattr(self, "shipping_address") else None)!r}, '
                f'payment_source={(self.payment_source if hasattr(self, "payment_source") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'shipping_address={(self.shipping_address if hasattr(self, "shipping_address") else None)!s}, '
                f'payment_source={(self.payment_source if hasattr(self, "payment_source") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.money import Money
from paypalserversdk.models.plan_override import PlanOverride
from paypalserversdk.models.shipping_details import ShippingDetails


class ModifySubscriptionResponse(object):

    """Implementation of the 'Modify Subscription Response' model.

    The response to a request to update the quantity of the product or service
    in a subscription. You can also use this method to switch the plan and
    update the `shipping_amount` and `shipping_address` values for the
    subscription. This type of update requires the buyer's consent.

    Attributes:
        plan_id (str): The unique PayPal-generated ID for the plan.
        quantity (str): The quantity of the product or service in the
            subscription.
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        shipping_address (ShippingDetails): The shipping details.
        plan (PlanOverride): An inline plan object to customise the
            subscription. You can override plan level default attributes by
            providing customised values for the subscription in this object.
        plan_overridden (bool): Indicates whether the subscription has
            overridden any plan attributes.
        links (List[LinkDescription]): An array of request-related [HATEOAS
            links](/docs/api/reference/api-responses/#hateoas-links).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "plan_id": 'plan_id',
        "quantity": 'quantity',
        "shipping_amount": 'shipping_amount',
        "shipping_address": 'shipping_address',
        "plan": 'plan',
        "plan_overridden": 'plan_overridden',
        "links": 'links'
    }

    _optionals = [
        'plan_id',
        'quantity',
        'shipping_amount',
        'shipping_address',
        'plan',
        'plan_overridden',
        'links',
    ]

    def __init__(self,
                 plan_id=APIHelper.SKIP,
                 quantity=APIHelper.SKIP,
                 shipping_amount=APIHelper.SKIP,
                 shipping_address=APIHelper.SKIP,
                 plan=APIHelper.SKIP,
                 plan_overridden=APIHelper.SKIP,
                 links=APIHelper.SKIP):
        """Constructor for the ModifySubscriptionResponse class"""

        # Initialize members of the class
        if plan_id is not APIHelper.SKIP:
            self.plan_id = plan_id 
        if quantity is not APIHelper.SKIP:
            self.quantity = quantity 
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if shipping_address is not APIHelper.SKIP:
            self.shipping_address = shipping_address 
        if plan is not APIHelper.SKIP:
            self.plan = plan 
        if plan_overridden is not APIHelper.SKIP:
            self.plan_overridden = plan_overridden 
        if links is not APIHelper.SKIP:
            self.links = links 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        plan_id = dictionary.get("plan_id") if dictionary.get("plan_id") else APIHelper.SKIP
        quantity = dictionary.get("quantity") if dictionary.get("quantity") else APIHelper.SKIP
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        shipping_address = ShippingDetails.from_dictionary(dictionary.get('shipping_address')) if 'shipping_address' in dictionary.keys() else APIHelper.SKIP
        plan = PlanOverride.from_dictionary(dictionary.get('plan')) if 'plan' in dictionary.keys() else APIHelper.SKIP
        plan_overridden = dictionary.get("plan_overridden") if "plan_overridden" in dictionary.keys() else APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        # Return an object of this model
        return cls(plan_id,
                   quantity,
                   shipping_amount,
                   shipping_address,
                   plan,
                   plan_overridden,
                   links)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'plan_id={(self.plan_id if hasattr(self, "plan_id") else None)!r}, '
                f'quantity={(self.quantity if hasattr(self, "quantity") else None)!r}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!r}, '
                f'shipping_address={(self.shipping_address if hasattr(self, "shipping_address") else None)!r}, '
                f'plan={(self.plan if hasattr(self, "plan") else None)!r}, '
                f'plan_overridden={(self.plan_overridden if hasattr(self, "plan_overridden") else None)!r}, '
                f'links={(self.links if hasattr(self, "links") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'plan_id={(self.plan_id if hasattr(self, "plan_id") else None)!s}, '
                f'quantity={(self.quantity if hasattr(self, "quantity") else None)!s}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!s}, '
                f'shipping_address={(self.shipping_address if hasattr(self, "shipping_address") else None)!s}, '
                f'plan={(self.plan if hasattr(self, "plan") else None)!s}, '
                f'plan_overridden={(self.plan_overridden if hasattr(self, "plan_overridden") else None)!s}, '
                f'links={(self.links if hasattr(self, "links") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.payer_name import PayerName
from paypalserversdk.models.phone import Phone
from paypalserversdk.models.simple_postal_address_coarse_grained import SimplePostalAddressCoarseGrained


class PayerInformation(object):

    """Implementation of the 'Payer Information' model.

    The payer information.

    Attributes:
        account_id (str): The PayPal` customer account ID.
        email_address (str): The internationalized email address. Note: Up to
            64 characters are allowed before and 255 characters are allowed
            after the @ sign. However, the generally accepted maximum length
            for an email address is 254 characters. The pattern verifies that
            an unquoted @ sign exists.
        phone_number (Phone): The phone number, in its canonical international
            [E.164 numbering plan
            format](https://www.itu.int/rec/T-REC-E.164/en).
        address_status (str): The address status of the payer. Value is
            either: Y. Verified. N. Not verified.
        payer_status (str): The status of the payer. Value is `Y` or `N`.
        payer_name (PayerName): The name of the party.
        country_code (str): The [two-character ISO 3166-1
            code](/docs/integration/direct/rest/country-codes/) that
            identifies the country or region. Note: The country code for Great
            Britain is GB and not UK as used in the top-level domain names for
            that country. Use the `C2` country code for China worldwide for
            comparable uncontrolled price (CUP) method, bank card, and
            cross-border transactions.
        address (SimplePostalAddressCoarseGrained): A simple postal address
            with coarse-grained fields. Do not use for an international
            address. Use for backward compatibility only. Does not contain
            phone.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "account_id": 'account_id',
        "email_address": 'email_address',
        "phone_number": 'phone_number',
        "address_status": 'address_status',
        "payer_status": 'payer_status',
        "payer_name": 'payer_name',
        "country_code": 'country_code',
        "address": 'address'
    }

    _optionals = [
        'account_id',
        'email_address',
        'phone_number',
        'address_status',
        'payer_status',
        'payer_name',
        'country_code',
        'address',
    ]

    def __init__(self,
                 account_id=APIHelper.SKIP,
                 email_address=APIHelper.SKIP,
                 phone_number=APIHelper.SKIP,
                 address_status=APIHelper.SKIP,
                 payer_status=APIHelper.SKIP,
                 payer_name=APIHelper.SKIP,
                 country_code=APIHelper.SKIP,
                 address=APIHelper.SKIP):
        """Constructor for the PayerInformation class"""

        # Initialize members of the class
        if account_id is not APIHelper.SKIP:
            self.account_id = account_id 
        if email_address is not APIHelper.SKIP:
            self.email_address = email_address 
        if phone_number is not APIHelper.SKIP:
            self.phone_number = phone_number 
        if address_status is not APIHelper.SKIP:
            self.address_status = address_status 
        if payer_status is not APIHelper.SKIP:
            self.payer_status = payer_status 
        if payer_name is not APIHelper.SKIP:
            self.payer_name = payer_name 
        if country_code is not APIHelper.SKIP:
            self.country_code = country_code 
        if address is not APIHelper.SKIP:
            self.address = address 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        account_id = dictionary.get("account_id") if dictionary.get("account_id") else APIHelper.SKIP
        email_address = dictionary.get("email_address") if dictionary.get("email_address") else APIHelper.SKIP
        phone_number = Phone.from_dictionary(dictionary.get('phone_number')) if 'phone_number' in dictionary.keys() else APIHelper.SKIP
        address_status = dictionary.get("address_status") if dictionary.get("address_status") else APIHelper.SKIP
        payer_status = dictionary.get("payer_status") if dictionary.get("payer_status") else APIHelper.SKIP
        payer_name = PayerName.from_dictionary(dictionary.get('payer_name')) if 'payer_name' in dictionary.keys() else APIHelper.SKIP
        country_code = dictionary.get("country_code") if dictionary.get("country_code") else APIHelper.SKIP
        address = SimplePostalAddressCoarseGrained.from_dictionary(dictionary.get('address')) if 'address' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(account_id,
                   email_address,
                   phone_number,
                   address_status,
                   payer_status,
                   payer_name,
                   country_code,
                   address)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'account_id={(self.account_id if hasattr(self, "account_id") else None)!r}, '
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!r}, '
                f'phone_number={(self.phone_number if hasattr(self, "phone_number") else None)!r}, '
                f'address_status={(self.address_status if hasattr(self, "address_status") else None)!r}, '
                f'payer_status={(self.payer_status if hasattr(self, "payer_status") else None)!r}, '
                f'payer_name={(self.payer_name if hasattr(self, "payer_name") else None)!r}, '
                f'country_code={(self.country_code if hasattr(self, "country_code") else None)!r}, '
                f'address={(self.address if hasattr(self, "address") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'account_id={(self.account_id if hasattr(self, "account_id") else None)!s}, '
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!s}, '
                f'phone_number={(self.phone_number if hasattr(self, "phone_number") else None)!s}, '
                f'address_status={(self.address_status if hasattr(self, "address_status") else None)!s}, '
                f'payer_status={(self.payer_status if hasattr(self, "payer_status") else None)!s}, '
                f'payer_name={(self.payer_name if hasattr(self, "payer_name") else None)!s}, '
                f'country_code={(self.country_code if hasattr(self, "country_code") else None)!s}, '
                f'address={(self.address if hasattr(self, "address") else None)!s})')
